<#

    .SYNOPSIS
    Simple uploader tool, used for SCCM Auto Management

    .DESCRIPTION

    .PARAMETER

    .EXAMPLE

    .NOTES
    Author: Morten Rønborg
    Date: 25-07-2018
    Last Updated: 30-09-2018
#>

################################################

try {
    $SourceLocation = (Import-Clixml -Path "$PSScriptRoot\configuration.config").SourceLocation
}
catch {
    
}

$LogLocation = $env:TEMP
$LogName = "SCCM-AutoManagement"
$Global:SyncHash = [hashtable]::Synchronized(@{})
$NewRunspace =[runspacefactory]::CreateRunspace()
$NewRunspace.ApartmentState = "STA"
$NewRunspace.ThreadOptions = "ReuseThread"
$NewRunspace.Open()
$NewRunspace.SessionStateProxy.SetVariable("JobCleanup",$JobCleanup)     
$NewRunspace.SessionStateProxy.SetVariable("Jobs",$Jobs) 
$NewRunspace.SessionStateProxy.SetVariable("SyncHash",$SyncHash)

#Dot source
. "$PSScriptRoot\lib.ps1"

#Define runspace
$psCmd = [PowerShell]::Create().AddScript({
    param(
        $SourceLocation,
        $ScriptRoot,
        $LogLocation,
        $LogName
    )

    #Dot source
    . "$ScriptRoot\lib.ps1"

    #Write log
    Write-Log -LogOutput ("Started runspace containing the form..") -FunctionName "Form runspace" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

    #Define the XAML
    $MainWindowXAML = Get-MainWindowXAML

    #Read the XAML
    $MainWindowReader = (New-Object System.Xml.XmlNodeReader $MainWindowXAML)
    
    #Define the shared runspace hastable
    $SyncHash.Window = [Windows.Markup.XamlReader]::Load($MainWindowReader)

    #Add to the hastable
    $MainWindowXAML.SelectNodes("//*[@*[contains(translate(name(.),'n','N'),'Name')]]") | ForEach-Object{
        
        #Find all of the form types and add them as members to the SyncHash
        $SyncHash.Add($_.Name,$SyncHash.Window.FindName($_.Name) )
    }

    #Define variables for use in various runspaces
    $SyncHash.Add("RoboScanID","")
    $SyncHash.Add("RoboRunID","")
    $SyncHash.Add("CancelOverwrite","")
    $SyncHash.Add("CancelUpload","")

    #Define cleanup variables
    $Script:JobCleanup = [hashtable]::Synchronized(@{})
    $Script:Jobs = [system.collections.arraylist]::Synchronized((New-Object System.Collections.ArrayList))

    #Region Background runspace to clean up jobs
    $JobCleanup.Flag = $True
    $NewRunspace =[runspacefactory]::CreateRunspace()
    $NewRunspace.ApartmentState = "STA"
    $NewRunspace.ThreadOptions = "ReuseThread"          
    $NewRunspace.Open()        
    $NewRunspace.SessionStateProxy.SetVariable("JobCleanup",$JobCleanup)     
    $NewRunspace.SessionStateProxy.SetVariable("Jobs",$Jobs) 
    $JobCleanup.PowerShell = [PowerShell]::Create().AddScript({

        #Routine to handle completed runspaces
        Do {    
            Foreach($runspace in $Jobs) {            
                If ($runspace.Runspace.isCompleted) {
                    [void]$runspace.powershell.EndInvoke($runspace.Runspace)
                    $runspace.powershell.dispose()
                    $runspace.Runspace = $null
                    $runspace.powershell = $null               
                } 
            }
            #Clean out unused runspace Jobs
            $temphash = $Jobs.clone()
            $temphash | Where {
                $_.runspace -eq $Null
            } | ForEach {
                $Jobs.remove($_)
            }        
            Start-Sleep -Seconds 1     
        } while ($JobCleanup.Flag)
    })
    $JobCleanup.PowerShell.Runspace = $NewRunspace
    $JobCleanup.Thread = $JobCleanup.PowerShell.BeginInvoke()  
    #endregion Background runspace to clean up jobs


    Write-Log -LogOutput ("$SourceLocation.") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

    #Get the current 
    # $ApplicationsInSource = (Import-ApplicationInfoFiles -Path $SourceLocation | Where-Object {($_.AutoManageAction -eq "Create") -or ($_.AutoManageState -eq "Created")})

    #When the upload buttion is clicked
    $SyncHash.btn_upload.Add_Click({

        # Define the new runspace
        $NewRunspace =[runspacefactory]::CreateRunspace()
        $NewRunspace.ApartmentState = "STA"
        $NewRunspace.ThreadOptions = "ReuseThread"          
        $NewRunspace.Open()
        $NewRunspace.SessionStateProxy.SetVariable("SyncHash",$SyncHash) 
        $PowerShell = [PowerShell]::Create().AddScript({
            param(
                $SourceLocation,
                $localsource,
                $publisher,
                $name,
                $version,
                $installcommand,
                $uninstallcommand,
                $localizeddisplayname,
                $ScriptRoot,
                $LogLocation,
                $LogName
            )

        # Dot source
        . "$ScriptRoot\lib.ps1"

        # Variables
        $ApplicationLocation = ($SourceLocation +"\"+ $publisher +"\" + $name + "\" + $version)
        $ApplicationSourceLocation = ($SourceLocation +"\"+ $publisher +"\" + $name + "\" + $version + "\SourceFiles" )
        $SyncHash["CancelOverwrite"] = $null
        $SyncHash["CancelUpload"] = $null

        #Copy with progress
        function Copy-WithProgress {
            <#.LINK
            Stolen from(thanks buddy!):
            https://keithga.wordpress.com/2014/06/23/copy-itemwithprogress

            .NOTES
            By Keith S. Garner (KeithGa@KeithGa.com) - 6/23/2014
            With inspiration by Trevor Sullivan @pcgeek86

            #>
            [CmdletBinding()]
            param (
                    [Parameter(Mandatory = $true)]
                    [string] $Source
                , [Parameter(Mandatory = $true)]
                    [string] $Destination
            )
            $ScanLog  = [IO.Path]::GetTempFileName()
            $RoboLog  = [IO.Path]::GetTempFileName()
            $ScanArgs = "`"$Source`" `"$Destination`" /MIR /ndl /TEE /bytes /Log:$ScanLog /nfl /L".Split(" ")
            #$ScanArgs = "`"$Source`" `"$Destination`" /MIR /bytes /Log:$ScanLog /L /NC /NP /NDL /NJH /NJS".Split(" ")
            $RoboArgs = "`"$Source`" `"$Destination`" /MIR /ndl /TEE /bytes /Log:$RoboLog /NC".Split(" ")

            Update-Window -Control btn_upload -Property IsEnabled -Value $false

            # Launch Robocopy Processes
            write-verbose ("Robocopy Scan:`n" + ($ScanArgs -join " "))
            write-verbose ("Robocopy Full:`n" + ($RoboArgs -join " "))
            $ScanRun = start-process robocopy -PassThru -WindowStyle Hidden -ArgumentList $ScanArgs
            $RoboRun = start-process robocopy -PassThru -WindowStyle Hidden -ArgumentList $RoboArgs

            # Keep ID's if wishing to cancel
            $SyncHash["RoboScanID"] = $ScanRun.ID
            $SyncHash["RoboRunID"] = $RoboRun.ID

            # Parse Robocopy "Scan" pass
            $ScanRun.WaitForExit()
            $LogData = get-content $ScanLog
            $TotalFiles = [regex]::Match($LogData[-5],".+:\s+(\d+)\s+(\d+)").Groups[2].Value
            if ($ScanRun.ExitCode -ge 10)
            {
                $LogData|out-string|Write-Error
                throw "Robocopy $($ScanRun.ExitCode)"
            }
            $FileSize = [regex]::Match($LogData[-4],".+:\s+(\d+)\s+(\d+)").Groups[2].Value
            write-verbose ("Robocopy Bytes: $FileSize `n" +($LogData -join "`n"))

            # Monitor Full RoboCopy
            while (!$RoboRun.HasExited)
            {
                $LogData = get-content $RoboLog
                $Files = $LogData -match "^\s*(\d+)\s+(\S+)"
                $FileBeingProcessed = $Files[($Files.Length-1)].Split("\")[-1]
                $FileCount = $Files.Count -1
                if ($Files -ne $Null )
                {
                    $copied = ($Files[0..($Files.Length-2)] | %{$_.Split("`t")[-2]} | Measure -sum).Sum
                    if ($LogData[-1] -match "(100|\d?\d\.\d)\%")
                    {
                        write-progress Copy -ParentID $RoboRun.ID -percentComplete $LogData[-1].Trim("% `t") $LogData[-1]
                        $Copied += $Files[-1].Split("`t")[-2] /100 * ($LogData[-1].Trim("% `t"))

                        #Update the form
                        Update-Window -Control pgs_bar -Property Value -Value $LogData[-1].Trim("% `t")
                        Update-Window -Control lbl_info -Property Text -Value "$($LogData[-1])..processing file '$FileBeingProcessed'"
                        Write-Log -LogOutput ("$($LogData[-1])..processing file '$FileBeingProcessed'") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
                        
                    }
                    else
                    {
                        Update-Window -Control pgs_bar -Property Value -Value 100
                    }
                    $PercentComplete = [math]::min(100,(100*$Copied/[math]::max($Copied,$FileSize)))
                    #write-progress ROBOCOPY -ID $RoboRun.ID -PercentComplete $PercentComplete $Files[-1].Split("`t")[-1] 
                    Update-Window -Control pgs_bartop -Property Value -Value $PercentComplete
                    Update-Window -Control lbl_infotop -Property Text -Value "$PercentComplete%.. $FileCount of $TotalFiles transferred"
                }
            }

            #Update the form
            Update-Window -Control pgs_bartop -Property Value -Value 100
            Update-Window -Control pgs_bar -Property Value -Value 100
            Update-Window -Control lbl_infotop -Property Text -Value "100%..$TotalFiles of $TotalFiles. Transfer complete."
            Update-Window -Control lbl_info -Property Text -Value "100%.. Transfer complete." 
            Update-Window -Control btn_upload -Property IsEnabled -Value $true

            # Parse full RoboCopy pass results, and cleanup
            (get-content $RoboLog)[-11..-2] | out-string | Write-Verbose
            [PSCustomObject]@{ ExitCode = $RoboRun.ExitCode }
            Remove-Item $RoboLog, $ScanLog

        }
        #Define update window function
        Function Update-Window {
            Param (
                $Control,
                $Property,
                $Value,
                [switch]$AppendContent
            )

            # This is kind of a hack, there may be a better way to do this
            If ($Property -eq "Close") {
                $SyncHash.Window.Dispatcher.invoke([action]{$SyncHash.Window.Close()},"Normal")
                Return
            }

            # This updates the control based on the parameters passed to the function
            $SyncHash.$Control.Dispatcher.Invoke([action]{
                # This bit is only really meaningful for the TextBox control, which might be useful for logging progress steps
                If ($PSBoundParameters['AppendContent']) {
                    $SyncHash.$Control.AppendText($Value)
                } Else {
                    $SyncHash.$Control.$Property = $Value
                }
            }, "Normal")
        }
        
        Write-Log -LogOutput ("Check if there is already files, in '$ApplicationLocation'..") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        if((Get-ChildItem $ApplicationLocation)){

            #Give the user the option to cancel
            Update-Window -Control Window -Property IsEnabled -Value $false
            Update-Window -Control Window -Property Opacity -Value "0.3"
            Update-Window -Control pop_overwrite -Property IsOpen -Value $true


            Write-Log -LogOutput ("Already files on share $CurrentState, prompting user") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
            
            #Wait for the user to make a choice
            while ($SyncHash.CancelOverwrite -like $null) {

                #Sleep
                Start-Sleep 2
                Write-Log -LogOutput ("Waiting for user input...") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
            }

            #If the user choose to cancel
            If($SyncHash.CancelOverwrite -eq $True){
                Write-Log -LogOutput ("User choose to cancel the upload") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
                Break
            }
            Write-Log -LogOutput ("User choose to continue the upload") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
        }
        
        Write-Log -LogOutput ("Begin upload..$($SyncHash.pop_overwrite.IsOpen)") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
        

        #Open progress dialog and disable main window
        Update-Window -Control Window -Property IsEnabled -Value $false
        Update-Window -Control Window -Property Opacity -Value "0.3"
        Update-Window -Control pop_upload -Property IsOpen -Value $true

        # Copy-WithProgress -Source $SyncHash.tbx_applocalsource -Destination $SourceLocation. If success crete the file to trigger app creation
        Copy-WithProgress -Source $LocalSource -Destination $ApplicationSourceLocation
        if($SyncHash.CancelUpload -like $null){

            #Create file on share to trigger event listner, end of buttonclick runspace
            New-ApplicationInfoFile -Path $ApplicationLocation -Publisher $publisher -Name $name -Version $version -LocalizedDisplayName $localizeddisplayname -InstallCommand $installcommand -UninstallCommand $uninstallcommand
        }

        })

        #Call the runspace called by the buttonclick
        $PowerShell.Runspace = $NewRunspace
        [void]$Jobs.Add((
            [pscustomobject]@{
                PowerShell = $PowerShell
                Argument1 = $PowerShell.AddArgument($SourceLocation)
                Argument2 = $PowerShell.AddArgument($($SyncHash.tbx_applocalsource.Text))
                Argument3 = $PowerShell.AddArgument($($SyncHash.tbx_apppublisher.Text))
                Argument4 = $PowerShell.AddArgument($($SyncHash.tbx_appname.Text))
                Argument5 = $PowerShell.AddArgument($($SyncHash.tbx_appversion.Text))
                Argument6 = $PowerShell.AddArgument($($SyncHash.tbx_installcommand.Text))
                Argument7 = $PowerShell.AddArgument($($SyncHash.tbx_uninstallcommand.Text))
                Argument8 = $PowerShell.AddArgument($($SyncHash.tbx_localizeddisplayname.Text))
                Argument9 = $PowerShell.AddArgument($ScriptRoot)
                Argument10 = $PowerShell.AddArgument($LogLocation)
                Argument11 = $PowerShell.AddArgument($LogName)
                Runspace = $PowerShell.BeginInvoke()
            }
        ))
    })

    #region Window Close 
    $SyncHash.Window.Closing({

        #See if Robocopy is running
        If(Get-Process -Id $SyncHash.RoboRunID){
            Stop-Process -Id $SyncHash.RoboRunID -Force
        }
        If(Get-Process -Id $SyncHash.RoboScanID){
            Stop-Process -Id $SyncHash.RoboScanID -Force
        }

        #Stop runspaces
        $JobCleanup.Flag = $False

        #Stop all runspaces
        $JobCleanup.PowerShell.Dispose()
    })

    $SyncHash.cb_applications.Add_SelectionChanged({

        #Get the info from the aray 
        $selecteditem = $SyncHash.cb_applications.SelectedItem
        $selectedapp = $ApplicationsInSource |  Where-Object {$_.Name -eq $selecteditem}
        
        #populate to form
        $SyncHash.tbx_apppublisher.Text = $selectedapp.Publisher
        $SyncHash.tbx_appname.text = $selectedapp.SimpleName
        $SyncHash.tbx_appversion.Text = $selectedapp.Version
        $SyncHash.tbx_localizeddisplayname.text = $selectedapp.LocalizedDisplayName
        $SyncHash.tbx_installcommand.text = $selectedapp.InstallCommand
        $SyncHash.tbx_uninstallcommand.text = $selectedapp.UninstallCommand

    })

    #Upload cancel click
    $SyncHash.pop_btn_upload_cancel.Add_Click({
        
        # $SyncHash.Window.IsEnabled = $true
        # $SyncHash.Window.Opacity = "1"
        $SyncHash.pop_upload.IsOpen = $false

        #See if Robocopy is running
        If(Get-Process -Id $SyncHash.RoboRunID){
            Write-Log -LogOutput ("Stopping the robocopy process with process ID: $($SyncHash.RoboRunID)..") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
            Stop-Process -Id $SyncHash.RoboRunID -Force
            $SyncHash["CancelUpload"] = $true
        }
        If(Get-Process -Id $SyncHash.RoboScanID){
            Write-Log -LogOutput ("Stopping the robocopy scan process with process ID: $($SyncHash.RoboScanID)..") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null
            Stop-Process -Id $SyncHash.RoboScanID -Force
            $SyncHash["CancelUpload"] = $true
        }

        Get-LoadApplicaitonList -SourceLoacation $SourceLocation
    })

    #If overwrite cancel click
    $SyncHash.pop_btn_overwrite_continue.Add_Click({

        #Define cancel variable to use in the other runspace
        $SyncHash["CancelOverwrite"] = $false

        #Show windows and close overwrite
        $SyncHash.pop_overwrite.IsOpen = $false

    })

    #If overwrite cancel click
    $SyncHash.pop_btn_overwrite_cancel.Add_Click({

        #Define cancel variable to use in the other runspace
        $SyncHash["CancelOverwrite"] = $true

        #Show windows and close overwrite
        $SyncHash.Window.IsEnabled = $true
        $SyncHash.Window.Opacity = "1"
        $SyncHash.pop_overwrite.IsOpen = $false
    })

    #No connection prompt exit click
    $SyncHash.pop_btn_noconnection_exit.Add_Click({

        #Write log
        Write-Log -LogOutput ("No connection, closing form..") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        #Close the form
        $SyncHash.Window.Close() | Out-Null
    })

    #Browse click
    $SyncHash.btn_browse.Add_Click({

        [Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") | Out-Null
        [System.Windows.Forms.Application]::EnableVisualStyles()
        $browse = New-Object System.Windows.Forms.FolderBrowserDialog
        $browse.SelectedPath = "C:\"
        $browse.ShowNewFolderButton = $false
        $browse.Description = "Select the root directory of the application to upload"
        $browse.ShowDialog()
        $SyncHash.tbx_applocalsource.Text = $browse.SelectedPath

        #If there is a deploy application script and the 
        if ($AppContent = Get-Content -Path "$($browse.SelectedPath)\Deploy-Application.ps1") {

            #Define variables
            $appVendorString = "appVendor"
            $appNameString = "appName"
            $appVersionString = "appVersion"
            $formVendor = $SyncHash.tbx_apppublisher.text
            $formName = $SyncHash.tbx_appname.text

    
            $appVendor =   ((($AppContent | Select-String $appVendorString)[0] -split "'")[1] -split "'")[0]
            $appName =   ((($AppContent | Select-String $appNameString)[0] -split "'")[1] -split "'")[0]
            $appVersion =   ((($AppContent | Select-String $appVersionString)[0] -split "'")[1] -split "'")[0]
            $appLocalizedName = $appName

            if(($formVendor -ne "") -and ($formName -ne "")) {

                #Prompt the user
                if (($formVendor -ne $appVendor) -or ($formName -ne $appName)) {
                    #Show-MessageBox -Message "The appname and/or vendor variables in the script '$($browse.SelectedPath)\Deploy-Application.ps1' does not match the current form input data.`n If this is not a new application, then check the variables in the Deploy-Application script.." -Type "Warning" | Out-Null
                }

                #If there is no input in vendor or name then we pressume its a new app, populate data
                $SyncHash.tbx_apppublisher.text =  $appVendor
                $SyncHash.tbx_appname.text = $appName
                $SyncHash.tbx_appversion.Text =  $appVersion
                $SyncHash.tbx_localizeddisplayname.text = $appLocalizedName
            }else{

                #If there is no input in vendor or name then we pressume its a new app, populate data
                $SyncHash.tbx_apppublisher.text =  $appVendor
                $SyncHash.tbx_appname.text = $appName
                $SyncHash.tbx_appversion.Text =  $appVersion
                $SyncHash.tbx_localizeddisplayname.text = $appLocalizedName
            }
        }
    })

    #Hyperlink
    $SyncHash.hpr_link.Add_Click({

        #Go to wonderland
        [system.Diagnostics.Process]::start("https://mroenborg.com")
    })

    #Close form
    $SyncHash.btn_close.Add_Click({

        #Write log
        Write-Log -LogOutput ("Closing form..") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        #Close the form
        $SyncHash.Window.Close() | Out-Null
    })

    #Close form
    $SyncHash.btn_delete.Add_Click({

        #Show window to delete
        $SyncHash.Window.IsEnabled = $true
        $SyncHash.Window.Opacity = "0.3"
        $SyncHash.pop_delete_yes_no.IsOpen = $true
        
    })
    $SyncHash.pop_btn_delete_no.Add_Click({

        #Hide window to delete
        $SyncHash.Window.IsEnabled = $true
        $SyncHash.Window.Opacity = "1"
        $SyncHash.pop_delete_yes_no.IsOpen = $false
    })
    #when delete continue is selected
    $SyncHash.pop_btn_delete_yes.Add_Click({

        #Get the filepath 
        $XMLFilePath = ($ApplicationsInSource | Where-Object {$_.Name -eq $($SyncHash.cb_applications.SelectedValue)}).XMLFilePath

        #Write log
        Write-Log -LogOutput ("Setting $($SyncHash.cb_applications.SelectedValue) $($SyncHash.tbx_appversion.Text) for the script to delete on the server '$XMLFilePath'") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        #Update the file to trigger the script to delete
        Update-ApplicationInfoFile -Path $XMLFilePath -PropertyName "AutoManageAction" -Value "Delete"

        #Load applications
        Get-LoadApplicaitonList -SourceLoacation $SourceLocation
    })

    #Drag window functionality
    $SyncHash.brd_top.Add_MouseLeftButtonDown({
        $SyncHash.Window.DragMove()
    })

    # #Add list of applications to the drop down
    # foreach($App in $ApplicationsInSource){
    #     $SyncHash.cb_applications.Items.Add(($App.Name))
    # }
    
    #Disable form objects
    $SyncHash.tbx_apppublisher.IsEnabled = $false
    $SyncHash.tbx_appname.IsEnabled = $false
    $SyncHash.tbx_appversion.IsEnabled = $false
    $SyncHash.tbx_applocalsource.IsEnabled = $false

    Get-LoadApplicaitonList -SourceLoacation $SourceLocation
    
    #Show the window
    $SyncHash.Window.ShowDialog() | Out-Null
    $SyncHash.Window.Topmost = $false
    $SyncHash.Error = $Error

})


$psCmd.Runspace = $NewRunspace
$psCmd.AddArgument($SourceLocation)
$psCmd.AddArgument($PSScriptRoot)
$psCmd.AddArgument($LogLocation)
$psCmd.AddArgument($LogName)
$Data = $psCmd.BeginInvoke()

#Wait for the runspace to finish
While(!($data.IsCompleted)){
    start-sleep 5
}