<#

    .SYNOPSIS
    Library file

    .DESCRIPTION

    .PARAMETER

    .EXAMPLE

    .NOTES
    Author: Morten Rønborg
    Date: 29-09-2018
    Last Updated: 29-09-2018

#>

################################################

#Add assemblys
Add-Type -AssemblyName PresentationFramework
Add-Type -AssemblyName System.Windows.Forms
Add-Type -Path $PSScriptRoot\MaterialDesignThemes.Wpf.dll
Add-Type -Path $PSScriptRoot\MaterialDesignColors.dll
Add-Type -Path $PSScriptRoot\PresentationFramework.dll
Function Write-Log
{
    param 
    (
        [Parameter(Mandatory=$true, HelpMessage="Provide a message")][string]$LogOutput,
        [Parameter(Mandatory=$true, HelpMessage="Provide the function name")][string]$FunctionName,
        [Parameter(Mandatory=$false, HelpMessage="Provide the scriptlinenumber")][string]$ScriptLine,
        [Parameter(Mandatory=$false, HelpMessage="Provide path, default is .\Logs")][string]$Path,
        [Parameter(Mandatory=$false, HelpMessage="Provide name for the logs")][string]$Name,
        [Parameter(Mandatory=$false, HelpMessage="Provide level, 1 = default, 2 = warning 3 = error")][ValidateSet(1, 2, 3)][int]$LogLevel = 1
    )

    #If the scriptline is not defined then use from the invocation
    If(!($ScriptLine)){
        $ScriptLine = $($MyInvocation.ScriptLineNumber)
    }

    if($LogOutput){

        #Date for the lognaming
        $FullLogName = ($Path + "\" + $Name + ".log")
        $FullSecodaryLogName = ($FullLogName).Replace(".log",".lo_")

        #If the log has reached over 20 mb then rename it
        if(Test-Path $FullLogName){
            if((Get-Item $FullLogName).Length -gt 5000kb){
                if(Test-Path $FullSecodaryLogName){
                    Remove-Item -Path $FullSecodaryLogName -force
                }
                Rename-Item -Path $FullLogName -NewName $FullSecodaryLogName
            }
        }

        #First check if folder/logfile excist, if not then create it
        if(!(test-path $Path)){
            New-Item -ItemType Directory -Force -Path $Path -ErrorAction SilentlyContinue
        }

        #Get current date and time to write to log
        $TimeGenerated = "$(Get-Date -Format HH:mm:ss).$((Get-Date).Millisecond)+000"

        #Construct the logline format
        $Line = '<![LOG[{0}]LOG]!><time="{1}" date="{2}" component="{3}" context="" type="{4}" thread="" file="">'

        #Define line
        $LineFormat = $logOutput, $TimeGenerated, (Get-Date -Format MM-dd-yyyy), "$($FunctionName):$($Scriptline)", $LogLevel

        #Append line
        $Line = $Line -f $LineFormat

        #Write log
        try {
            Write-Host ("[$($FunctionName):$($Scriptline)]" + $logOutput)
            $Line | Out-File -FilePath ($Path + "\" + $Name + ".log") -Append -NoClobber -Force -Encoding 'UTF8' -ErrorAction 'Stop'
        }
        catch {
            Write-Host "$_"
        }
    }
}
Function Get-MainWindowXAML{

    #Define the XAML
    [xml]$XAML = @"
    <Window x:Class="MainWindow"
    TextElement.Foreground="{DynamicResource MaterialDesignBody}"
    Background="{DynamicResource MaterialDesignPaper}"
    xmlns="http://schemas.microsoft.com/winfx/2006/xaml/presentation"
    xmlns:x="http://schemas.microsoft.com/winfx/2006/xaml"
    xmlns:d="http://schemas.microsoft.com/expression/blend/2008"
    xmlns:mc="http://schemas.openxmlformats.org/markup-compatibility/2006"
    xmlns:materialDesign="http://materialdesigninxaml.net/winfx/xaml/themes"
    xmlns:local="clr-namespace:SCCM_Auto_Management_Uploader"
    mc:Ignorable="d" 
    FontSize="15"
    Title="SCCM Auto Management" Height="600" Width="550" WindowStartupLocation="CenterScreen" ResizeMode="NoResize" ShowInTaskbar="True" Topmost="False" AllowsTransparency="True" WindowStyle="None">
<Window.Resources>
    <ResourceDictionary>
        <ResourceDictionary.MergedDictionaries>
            <ResourceDictionary Source="pack://application:,,,/MaterialDesignThemes.Wpf;component/Themes/MaterialDesignTheme.Light.xaml" />
            <ResourceDictionary Source="pack://application:,,,/MaterialDesignThemes.Wpf;component/Themes/MaterialDesignTheme.Defaults.xaml" />
            <ResourceDictionary Source="pack://application:,,,/MaterialDesignColors;component/Themes/Recommended/Primary/MaterialDesignColor.Blue.xaml" />
            <ResourceDictionary Source="pack://application:,,,/MaterialDesignColors;component/Themes/Recommended/Accent/MaterialDesignColor.Lime.xaml" />
        </ResourceDictionary.MergedDictionaries>
    </ResourceDictionary>
</Window.Resources>

<Grid>
    <Grid.RowDefinitions>
        <RowDefinition Height="30"/>
        <RowDefinition Height="100"/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition/>
        <RowDefinition Height="35"/>
        <RowDefinition Height="20"/>
    </Grid.RowDefinitions>
    <Border Grid.Row="0" x:Name="brd_top" Background="Transparent">
        <Grid>
            <materialDesign:ColorZone Mode="PrimaryDark">
                <StackPanel HorizontalAlignment="Right">
                    <Button x:Name="btn_close" Width="30" Height="30" materialDesign:ShadowAssist.ShadowDepth="Depth3" Padding="0">
                        <materialDesign:PackIcon Kind="WindowClose"/>
                    </Button>
                </StackPanel>
            </materialDesign:ColorZone>
        </Grid>
    </Border>
    <Grid Grid.Row ="1">
        <Grid.ColumnDefinitions>
            <ColumnDefinition Width="82*"/>
            <ColumnDefinition Width="193*"/>
        </Grid.ColumnDefinitions>
        <materialDesign:ColorZone Mode="PrimaryMid" Grid.ColumnSpan="2" Margin="0,0,0.4,0">
            <TextBlock Text="SCCM Auto Management"  FontSize="35" HorizontalAlignment="Left" VerticalAlignment="Bottom" Margin="30 50 0 0"/>
        </materialDesign:ColorZone>
    </Grid>
    <Grid Grid.Row="2">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="Package" Foreground="{Binding ElementName=cb_applications,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <ComboBox x:Name="cb_applications" materialDesign:HintAssist.Hint="Choose excisting application for reference" materialDesign:HintAssist.IsFloating="True" Width="420"/>
                <Button x:Name="btn_delete" Content="Delete" Style="{DynamicResource MaterialDesignFlatButton}" Margin="5 5 5 5" VerticalAlignment="Bottom" HorizontalAlignment="Right"/>

            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="3">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="FolderOpen" Foreground="{Binding ElementName=tbx_applocalsource,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_applocalsource" materialDesign:HintAssist.Hint="Path" materialDesign:HintAssist.IsFloating="True" Width="420"/>
                <Button x:Name="btn_browse" Content="Browse" Style="{DynamicResource MaterialDesignFlatButton}" Margin="5 5 5 5" VerticalAlignment="Bottom" HorizontalAlignment="Right"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="4">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="Factory" Foreground="{Binding ElementName=tbx_apppublisher,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_apppublisher" materialDesign:HintAssist.Hint="Manufacturer/Vendor" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="5">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="RenameBox" Foreground="{Binding ElementName=tbx_appname,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_appname" materialDesign:HintAssist.Hint="Application name" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="6">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="Numeric" Foreground="{Binding ElementName=tbx_appversion,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_appversion" materialDesign:HintAssist.Hint="Version" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="7">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="RenameBox" Foreground="{Binding ElementName=tbx_localizeddisplayname,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_localizeddisplayname" materialDesign:HintAssist.Hint="Software Center name" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="8">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="RunFast" Foreground="{Binding ElementName=tbx_installcommand,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_installcommand" materialDesign:HintAssist.Hint="Install commandline" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="9">
        <StackPanel Orientation="Vertical">
            <StackPanel Orientation="Horizontal">
                <materialDesign:PackIcon Width="30" Kind="BackupRestore" Foreground="{Binding ElementName=tbx_uninstallcommand,Path=BorderBrush}" VerticalAlignment="Bottom" Margin="0 4"/>
                <TextBox x:Name="tbx_uninstallcommand" materialDesign:HintAssist.Hint="Uninstall commandline" materialDesign:HintAssist.IsFloating="True" Width="500"/>
            </StackPanel>
        </StackPanel>
    </Grid>
    <Grid Grid.Row="10" Width="525">
        <Button x:Name="btn_upload" Content="Upload" Margin="0 0 5 0" VerticalAlignment="Center" HorizontalAlignment="Right" Style="{DynamicResource MaterialDesignFlatButton}"/>
    </Grid>
    <Grid Grid.Row="11" Margin="5 0 0 0" VerticalAlignment="Top">
        <TextBlock>
        <Hyperlink x:Name="hpr_link" NavigateUri="http://www.mroenborg.com">
            mroenborg.com
        </Hyperlink>
        </TextBlock>
    </Grid>
    <materialDesign:DialogHost x:Name="pop_upload" HorizontalAlignment="Center" VerticalAlignment="Center" IsOpen="False" Grid.RowSpan="6">
        <materialDesign:DialogHost.DialogContent>
            <StackPanel Margin="16" Orientation="Vertical" Width="400">
                <Label Content="Upload Progress" FontSize="16" FontWeight="Bold" />
                <StackPanel Orientation="Vertical">
                    <TextBlock TextWrapping="Wrap" Text="When the upload is complete, the application creation will be done by the scheduled task on the server" FontSize="14"/>
                </StackPanel>
                <ProgressBar x:Name="pgs_bartop" Height="12" VerticalAlignment="Center" Margin="5 5 5 5" Width="Auto"/>
                <TextBlock x:Name="lbl_infotop"  HorizontalAlignment="Left" VerticalAlignment="Center" FontSize="10" Margin="7 5 5 5"/>
                <ProgressBar x:Name="pgs_bar"  Height="12" VerticalAlignment="Center" Margin="5 5 5 5" Width="Auto"/>
                <TextBlock x:Name="lbl_info" HorizontalAlignment="Left" VerticalAlignment="Center" FontSize="10" Margin="7 5 5 5"/>
                <Button x:Name="pop_btn_upload_cancel" HorizontalAlignment="Center" VerticalAlignment="Center" Content="Cancel/Exit" Style="{DynamicResource MaterialDesignFlatButton}"/>
            </StackPanel>
        </materialDesign:DialogHost.DialogContent>
    </materialDesign:DialogHost>
    <materialDesign:DialogHost x:Name="pop_noconnection" HorizontalAlignment="Center" VerticalAlignment="Center" IsOpen="false" Grid.RowSpan="6">
        <materialDesign:DialogHost.DialogContent>
            <StackPanel Margin="16" Orientation="Vertical" Width="400">
                <Label Content="Error" FontSize="16" FontWeight="Bold"/>
                <StackPanel Orientation="Vertical">
                    <TextBlock TextWrapping="Wrap" Text="No connection or missing permissions to management share" FontSize="14"/>
                </StackPanel>
                    <Button x:Name="pop_btn_noconnection_exit" HorizontalAlignment="Center" VerticalAlignment="Center" Content="Exit" Style="{DynamicResource MaterialDesignFlatButton}"/>
            </StackPanel>
        </materialDesign:DialogHost.DialogContent>
    </materialDesign:DialogHost>
    <materialDesign:DialogHost x:Name="pop_overwrite" HorizontalAlignment="Center" VerticalAlignment="Center" IsOpen="false" Grid.RowSpan="6">
        <materialDesign:DialogHost.DialogContent>
            <StackPanel Margin="16" Orientation="Vertical" Width="400">
                <Label Content="Warning" FontSize="16" FontWeight="Bold"/>
                <StackPanel Orientation="Vertical">
                    <TextBlock TextWrapping="Wrap" Text="There is already excisting content for this specific application. Do you want to overwrite the content on the share, mirroring the data from the given path?" FontSize="14"/>
                </StackPanel>
                <StackPanel Orientation="Horizontal" HorizontalAlignment="Center">
                    <Button x:Name="pop_btn_overwrite_continue" HorizontalAlignment="Center" VerticalAlignment="Center" Content="Continue" Style="{DynamicResource MaterialDesignFlatButton}"/>
                    <Button x:Name="pop_btn_overwrite_cancel" HorizontalAlignment="Center" VerticalAlignment="Center" Content="Exit" Style="{DynamicResource MaterialDesignFlatButton}"/>
                </StackPanel>
            </StackPanel>
        </materialDesign:DialogHost.DialogContent>
    </materialDesign:DialogHost>
    <materialDesign:DialogHost x:Name="pop_delete_yes_no" HorizontalAlignment="Center" VerticalAlignment="Center" IsOpen="false" Grid.RowSpan="6">
        <materialDesign:DialogHost.DialogContent>
            <StackPanel Margin="16" Orientation="Vertical" Width="400">
                <Label Content="Confirm" FontSize="16" FontWeight="Bold"/>
                <StackPanel Orientation="Vertical">
                    <TextBlock TextWrapping="Wrap" Text="Are you sure you want to delete the selected appliation? This will not delete the sourcefiles on the share" FontSize="14"/>
                </StackPanel>
                <StackPanel Orientation="Horizontal" HorizontalAlignment="Center">
                    <Button x:Name="pop_btn_delete_yes" HorizontalAlignment="Center" VerticalAlignment="Center" Content="Yes" Style="{DynamicResource MaterialDesignFlatButton}"/>
                    <Button x:Name="pop_btn_delete_no" HorizontalAlignment="Center" VerticalAlignment="Center" Content="No" Style="{DynamicResource MaterialDesignFlatButton}"/>
                </StackPanel>
            </StackPanel>
        </materialDesign:DialogHost.DialogContent>
    </materialDesign:DialogHost>
    <materialDesign:DialogHost x:Name="pop_loading" HorizontalAlignment="Center" VerticalAlignment="Center" IsOpen="False" Grid.RowSpan="6">
        <materialDesign:DialogHost.DialogContent>
            <StackPanel Margin="16" Orientation="Vertical" Width="400">
                <Label Content="Loading" FontSize="16" FontWeight="Bold"/>
                <StackPanel Orientation="Vertical">
                    <TextBlock TextWrapping="Wrap" Text="Please wait.." FontSize="14"/>
                </StackPanel>
                <StackPanel Orientation="Horizontal" HorizontalAlignment="Center">
                </StackPanel>
            </StackPanel>
        </materialDesign:DialogHost.DialogContent>
    </materialDesign:DialogHost>
</Grid>
</Window>
"@

#Remove attributes
$CleanedXAML = Get-CleanXAML -XAML $XAML

#Return
Return $CleanedXAML

}

Function Get-CleanXAML{
    param(
        $XAML
    )

    #Attributes to remove
    $AttributesToRemove = @(
        'x:Class',
        'mc:Ignorable'
    )

    #Remove XML attributes that breaks PS
    foreach ($Attrib in $AttributesToRemove) {
        if ( $XAML.Window.GetAttribute($Attrib) ) {
                $XAML.Window.RemoveAttribute($Attrib)
        }
    }
    # [xml]$Form = $XAML -replace 'mc:Ignorable="d"','' -replace "x:N",'N'  -replace '^<Win.*', '<Window'
    #Return cleaned XML
    Return $XAML
}
Function Get-ChildItemToDepth {
    Param(
    [string]$Path,
    [string]$Filter,
    [int]$ToDepth = 3,
    [int]$CurrentDepth
    )
    $CurrentDepth++
    Get-ChildItem $Path -Filter $Filter | ForEach-Object{
        if($_.FullName -like $Filter){
            [array]$PathToFile += $_.FullName
            }
        If ($_.PsIsContainer) {
            If ($CurrentDepth -le $ToDepth){
                # Callback to this function
                Get-ChildItemToDepth -Path $_.FullName -Filter $Filter -ToDepth $ToDepth -CurrentDepth $CurrentDepth
            }
        }
    }
    
    #return
    return $PathToFile
}
Function Import-ApplicationInfoFiles{
    Param(
        $Path
    )

    #Get autoapp files
    $AppsPath = Get-ChildItemToDepth -Path $Path -filter "*.*" -ToDepth 3 | Where-Object{$_ -like "*.autoapp"}

    #Define array
    $ApplicationInfo = @()
    
    #Put into array
    ForEach($AppPath in $AppsPath){

        #Import XML
        $AppInfo = Import-Clixml -Path "Microsoft.PowerShell.Core\FileSystem::$AppPath"
        $AppInfo | Add-Member -MemberType NoteProperty -Name "XMLFilePath" -Value  $AppPath -Force
        $ApplicationInfo += $AppInfo
    }

    #Return appinfo
    Return $ApplicationInfo
}

function New-ApplicationInfoFile {
    param(
        $Path,
        $Version,
        $Name,
        $Publisher,
        $LocalizedDisplayName,
        $InstallCommand,
        $UninstallCommand
    )

    #Define object
    $AppObj = New-Object -TypeName PSObject -Property @{

        #Add value to the object.
        SimpleName = $Name
        Name = ($Publisher + " " + $Name + " " +  $Version)
        LocalizedDisplayName = $LocalizedDisplayName
        Publisher = $Publisher
        Version = $Version
        SourcePath = "$Path\SourceFiles"
        IconLocation = "$Path\SourceFiles\$Version.ico"
        InstallCommand = $InstallCommand
        UninstallCommand = $UninstallCommand
        AutoManageAction = "Create"
        AutoManageState = ""
    }

    #Create the dir if not there
    New-Item -ItemType Directory -Path "$Path" -Force

    #Define the name
    $TrimmedName = ($Publisher + "_" + $Name + "_" + $($Version)).Replace(" ","")

    #Output the XML
    $AppObj | Export-Clixml -Path ($Path + "\" + $TrimmedName +  ".autoapp") -Force

}

function Update-ApplicationInfoFile {
    param(
        $Path,
        $PropertyName,
        $Value
    )

    #Appinfo
    $AppInfo = Import-Clixml -Path "Microsoft.PowerShell.Core\FileSystem::$Path"

    #Add the member
    $AppInfo | Add-Member -MemberType NoteProperty -Name $PropertyName -Value  $Value -Force

    #Output the XML
    $AppInfo | Export-Clixml -Path ("Microsoft.PowerShell.Core\FileSystem::" + $Path) -Force

}

function Test-MimimumAccessRights{
    param(
        $UserName,
        $Path
    )

    #Define variables
    $PSPath =("Microsoft.PowerShell.Core\FileSystem::" + $Path) 

    #Get all the memberships of the user specified
    [array]$GroupMemberShips = (New-Object System.DirectoryServices.DirectorySearcher("(&(objectCategory=User)(samAccountName=$UserName))")).FindOne().GetDirectoryEntry().memberOf

    #Get all FullControl and modify with inheritance
    $IdentityWithFullControlAndInheritance = ((Get-Acl $PSPath).Access | Where-Object{($_.InheritanceFlags -eq "ContainerInherit, ObjectInherit") -and (($_.FileSystemRights -eq "FullControl" -or $_.FileSystemRights -eq "Modify, Synchronize" -or $_.FileSystemRights -eq "DeleteSubdirectoriesAndFiles, Modify, Synchronize"))}).IdentityReference
   
    ForEach($Identity in $IdentityWithFullControlAndInheritance){

        #Define variables
        $FormattedIdentity = ($Identity).ToString().Split("\")[1]

        #Check if there is directly user access
        If($UserName -eq $FormattedIdentity){
            Return $true
        }

        #Check if there is a membership match
        ForEach($GroupMemberShip in $GroupMemberShips){

            #If there is a membership match return true
            If(($GroupMemberShip -like "*$FormattedIdentity*")-and ($FormattedIdentity -notlike $null)){
                Return $true
            }
        }
    }

    #return false
    Return $false
}
function Get-LoadApplicaitonList{
    param(
        $SourceLoaction
    )
    
    #Check rights to the share
    if(!(Test-MimimumAccessRights -Username $env:username -Path $SourceLocation)){

        #Show dialog
        $SyncHash.Window.IsEnabled = $false
        $SyncHash.Window.Opacity = "0.3"
        $SyncHash.pop_noconnection.IsOpen = $true
    
    }else{

        #Connectio to fileshare is met, show loading 
        $SyncHash.Window.IsEnabled = $false
        $SyncHash.Window.Opacity = "0.3"
        $SyncHash.pop_loading.IsOpen = $true
        Write-Log -LogOutput ("Loading applications from '$($SourceLocation)") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        #get applications
        $Global:ApplicationsInSource = (Import-ApplicationInfoFiles -Path $SourceLocation | Where-Object {(($_.AutoManageAction -eq "Create") -or ($_.AutoManageState -eq "Created")) -and !(($_.AutoManageAction -eq "Delete"))})
        
        #Add list of applications to the drop down
        $SyncHash.cb_applications.Items.Clear()
        foreach($App in $ApplicationsInSource){
                $SyncHash.cb_applications.Items.Add(($App.Name))
                Write-Log -LogOutput ("Loading application $($App.Name)") -FunctionName "Application Upload" -Path $Global:LogLocation -Name $Global:LogName | Out-Null

        }
        
        $SyncHash.cb_applications.items.Refresh()
        #Show window
        $SyncHash.Window.Opacity = "1"
        $SyncHash.Window.IsEnabled = $true
        $SyncHash.pop_loading.IsOpen = $false
        $SyncHash.pop_delete_yes_no.IsOpen = $false

    }
}

