<#

    .SYNOPSIS
    Simple filewatcher, used for SCCM Auto Management

    It should be running from a scheduled task by powershell with the current syntax for arguments:
    -ExecutionPolicy Bypass -windowstyle hidden -noexit -File "$PSScriptPath" -Argument $Argument1 -Argument2 $Argument2 ....

    .DESCRIPTION

    .PARAMETER

    .EXAMPLE
    -ExecutionPolicy Bypass -windowstyle hidden -noexit -File "\\Program Files(86)\SCCM Auto Management\SCCM-AutoManagementFileWatcher.ps1" -ScheduledTaskCreate "SCCM Auto Management" -ScheduledTaskDelete "SCCM Auto Management Delete" -Sourcefolder "\\Sccm\cm$\SD Auto Managed" 

    .NOTES
    Author: Morten Rønborg
    Date: 25-07-2018
    Last Updated: 25-07-2018

#>

################################################
param(
    $ScheduledTaskCreate = "SCCM Auto Management" ,
    # $ScheduledTaskDelete ,
    $SourceLocation
)

 # Dot Source 
. "$PSScriptRoot\lib.ps1"
##*=============================================
##* VARIABLES LISTINGS
##*=============================================

#Declare variables
$Filter = "*.autoapp"
$LogLocation = "$PSScriptRoot\Logs\" 
$LogName = "SCCM-AutoManagementFileWatcher"
$ScheduledTaskPath = "\SCCM Auto Management\"

##*=============================================
##* ENDVARIABLES LISTINGS
##*=============================================

Write-Log -LogOutput ("***********************************************START SCRIPT***********************************************") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null

#Declare filewatcher                  
$FileSystemWatcher = New-Object IO.FileSystemWatcher $SourceLocation, $Filter -Property @{IncludeSubdirectories = $true;NotifyFilter = [IO.NotifyFilters]'FileName, LastWrite'} 
Write-Log -LogOutput ("Starting filewatchers, monitoring the folder '$SourceLocation'.." ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null

#Create the filewatcher create
# Register-ObjectEvent $FileSystemWatcher Created -SourceIdentifier FileCreated -Action { 

#     #Declare variables
#     $Name = $Event.SourceEventArgs.Name 
#     $ChangeType = $Event.SourceEventArgs.ChangeType 
#     $TimeStamp = $Event.TimeGenerated
#     Write-Log -LogOutput ("The file '$Name' was '$ChangeType' at '$TimeStamp'. Starting scheduled task '$ScheduledTaskCreate'..." ) -ScriptLine $LogName -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null

#     #If the scheduled task is already running, then wait for it to be done
#     while ((Get-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskCreate).State -eq "Running") {
#         Write-Log -LogOutput ("The scheduled task is already running, waiting for it to be done..." ) -FunctionName $LogName -ScriptLine $LogName -Path $LogLocation -Name $LogName | Out-Null
#         start-sleep 5
#     }

#     #Start the task
#     try {
#         Write-Log -LogOutput ("Starting the scheduled task '$ScheduledTaskCreate'" ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
#         Start-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskCreate -ErrorAction Stop
#     }
#     catch {
#         Write-Log -LogOutput ("Failed to start the scheduled task '$ScheduledTaskCreate' with the error: $_" ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
#     }
# } 

#Create filewatcher change
Register-ObjectEvent $FileSystemWatcher Changed -SourceIdentifier FileChanged -Action { 
    
    #Declare variables
    $Name = $Event.SourceEventArgs.Name 
    $ChangeType = $Event.SourceEventArgs.ChangeType 
    $TimeStamp = $Event.TimeGenerated 
    Write-Log -LogOutput ("The file '$Name' was '$ChangeType' at '$TimeStamp'. Starting scheduled task '$ScheduledTaskCreate'..." ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null

    #If the scheduled task is already running, then wait for it to be done
    while ((Get-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskCreate).State -eq "Running") {
        Write-Log -LogOutput ("The scheduled task is already running, waiting for it to be done..." ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
        start-sleep 5
    }

    #Start the task
    try {
        Write-Log -LogOutput ("Starting the scheduled task '$ScheduledTaskCreate'" ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
        Start-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskCreate -ErrorAction Stop
    }
    catch {
        Write-Log -LogOutput ("Failed to start the scheduled task '$ScheduledTaskCreate' with the error: $_" ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
    }
} 
     
 
# #Create the filewatcher delete
# Register-ObjectEvent $FileSystemWatcher Deleted -SourceIdentifier FileDeleted -Action { 
    
#     #Declare variables
#     $Name = $Event.SourceEventArgs.Name 
#     $ChangeType = $Event.SourceEventArgs.ChangeType 
#     $TimeStamp = $Event.TimeGenerated 
#     Write-Log -LogOutput ("The file '$Name' was '$ChangeType' at '$TimeStamp'. Starting scheduled task '$ScheduledTaskDelete'..." ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null

#     #If the scheduled task is already running, then wait for it to be done
#     while ((Get-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskDelete).State -eq "Running") {
#         Write-Log -LogOutput ("The scheduled task is already running, waiting for it to be done..." ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
#         start-sleep 5
#     }

#     #Start the task
#     try {
#         #Start-ScheduledTask -TaskPath $ScheduledTaskPath -TaskName $ScheduledTaskDelete -ErrorAction Stop
#     }
#     catch {
#         Write-Log -LogOutput ("Failed to start the scheduled task '$ScheduledTaskDelete' with the error: $_" ) -FunctionName $LogName -Path $LogLocation -Name $LogName | Out-Null
#     }
# } 
 
# To stop the monitoring, run the following commands: 
# Unregister-Event FileDeleted 
# Unregister-Event FileCreated 
# Unregister-Event FileChanged